'use strict';

const debug = require('debug')('mongoose');
const mongoose = require('mongoose');

dependencies.mongodb = { api: false };

mongoose.set('debug', __config.databases.api.debug);

let connection = null;
let RECONNECT = true;

const connector = () => {
  debug('connecting ...');
  connection = mongoose.createConnection(__config.databases.api.uri, __config.databases.api.options, (err) => {
    if (err) return debug(err);
    global.db = connection;
  });
  return connection;
};

module.exports = connector();

connection.on('connected', () => {
  dependencies.mongodb.api = true;
  debug('connected');
});

connection.on('error', (err) => {
  dependencies.mongodb.api = false;
  debug(err);
});

connection.on('reconnected', () => {
  dependencies.mongodb.api = true;
  debug('reconnected');
});

connection.on('disconnected', () => {
  dependencies.mongodb.api = false;
  debug('disconnected, try to reconnect');
  if (RECONNECT) {
    setTimeout(connector, __config.databases.api.options.server.reconnect_timeout);
  }
});

process.on('SIGINT', () => {
  RECONNECT = false;
  dependencies.mongodb.api = false;
  connection.close(() => {
    debug('closed, exit node process.');
    process.exit(0);
  });
});

require('./schemas/signal');
require('./schemas/data');
