'use strict';
const ITEMS_PER_PAGE = 25;

const ambMap = {
  'g4-devel': 'Intelisis_Dev',
  'g4-qa': 'Intelisis_QA',
  'g4-prod': 'Intelisis_REL',
};

const ambName = ambMap[process.env.NODE_ENV] || 'Intelisis_Dev';

exports.getCatalogsPerPage = (options) => {
  const { kernelIp, tenantName, fields, catalog, page } = options;
  return new Promise((resolve, reject) => {
    const url = `/subtypes/records?tenant_id=${tenantName}&fields=${fields}&subType=${catalog}&page=${page}&limit=${ITEMS_PER_PAGE}`;
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.get(url, (err, response, body) => {
      if (err) return reject(err);
      if (body.status !== 200) return reject(new Error('Kernel Error'));
      resolve(body.response);
    });
  });
};

exports.getCatalogs = (options) => {
  const { kernelIp, tenantName } = options;
  return new Promise((resolve, reject) => {
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.get(`/subtypes?tenant_id=${tenantName}&type=entity`, (err, response, body) => {
      if (err) return reject(err);
      if (body.status !== 200) return reject(new Error('Kernel Error'));
      resolve(body.response);
    });
  });
};

exports.createOptions = (actions) => {
  actions.map((action) => {
    const { label, icon, callback } = action;
    return {
      status: 'on',
      icon,
      label,
      callback,
    };
  });
  return {
    right: [{
      icon: 'actionbar-more',
      className: 'more',
      status: 'on',
      actions,
    }],
  };
};

