'use strict';
const _ = require('underscore');
/**
 * Gets catalogs form kernel
 * @param  {Object} options
 * @return [Array]
 */
exports.getCatalogIds = options => {
  const { kernelIp, data } = options;
  return new Promise((resolve, reject) => {
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.post('/kernelfunctions/catid', data, (err, response, body) => {
      const status = body.status;
      if (err) return reject(err);
      if (status !== 200) return reject(JSON.stringify(body));
      return resolve(body.response.data);
    });
  });
};

exports.mergeCodes = (catalogs, codes) => {
  // console.log(JSON.stringify(catalogs, null, 2));
  const columns = [];
  _.mapObject(catalogs, (catalogCodes, subType) => {
    const columnCodes = [];
    catalogCodes.map(catalog => {
      codes.map(code => {
        if (code[catalog.name]) {
          columnCodes.push(code[catalog.name]);
        }
        return code;
      });
      return catalog;
    });
    const column = {
      codes: columnCodes,
      subType,
    };
    columns.push(column);
  });
  return columns;
};
/**
 * Gets the catalogs names.
 *
 * @param      [Array]  sections  The sections
 * @return     [Array]  The catalogs names.
 */
exports.getCatalogsNames = sections => {
  const catalogsName = {};
  sections.map(section => {
    if (section.hasOwnProperty('helper')) {
      const catalogName = section.helper.options.catalog;
      if (section.helper.type === 'catalog') {
        const catalogData = {
          // catalogName: section.helper.options.catalog,
          name: section.name,
        };
        if (!catalogsName[catalogName]) catalogsName[catalogName] = [];
        catalogsName[catalogName].push(catalogData);
      }
    }
    return section;
  });
  return catalogsName;
};

exports.mergeData = (ids, codes) => {
  const results = [];
  // console.log(JSON.stringify(ids, null, 2));
  ids.map(id => {
    _.mapObject(id, (values, subtype) => {
      id[subtype].map(idItems => {
        codes.map(code => {
          _.mapObject(code, (catalogCode, name) => {
            if (catalogCode === idItems.code) {
              let codeKernel = undefined;
              id[name].map(codeId => {
                if (codeId.code === code[name]) {
                  codeKernel = codeId;
                }
              });
              code[name] = codeKernel;
              results.push(code);
            }
          });
        });
      });
    });
  });
  return results;
};

exports.mergeCodesIds = (codesIds, codes) => {
  return codes.map(code => {
    codesIds.map(codeId => {
      if (code.code === codeId.code) {
        code._id = codeId._id;
      }
    });
    return code;
  });
};
