/**
 * [request: HTTP request client]
 */
const request = require('request');

/**
 * [req: fucntion to make request to the kernel]
 * @param  {String} functionName
 * @param  {Object} options
 * @param  {String} kernelIp
 * @return {Object}              Kernel response
 */
const req = (functionName, options, kernelIp) => {
  return new Promise((resolve, reject) => {
    const method = options ? 'POST' : 'GET';
    const url = `http://${kernelIp}/kernelfunctions/${functionName}`;

    return request({
      url,
      json: true,
      body: options,
      method,
    }, (err, body, response) => {
      if (err) return reject(err);
      if (response.status !== 200) {
        const kernelError = { kernelError: response, url };
        return reject(kernelError);
      }
      return resolve(response.response, body);
    });
  });
};

exports.request = req;
