'use strict';


const ambMap = {
  'g4-devel': 'Intelisis_Dev',
  'g4-qa': 'Intelisis_QA',
  'g4-prod': 'Intelisis_REL',
};

const ambName = ambMap[process.env.NODE_ENV] || 'Intelisis_Dev';
const getDashboardURLHTTP = require('../reports/functions').getDashboardURLHTTP;
const request = require('request');

/**
 * [abstractions: Useful functions]
 * @type {Object}
 */
const abstractions = require('./abstractions');

 /**
  * [kernelFunctionsWS: Get kernel functions by socket request]
  */
exports.kernelFunctionsWS = function kernelFunctions(socket, io, packet) {
  const options = packet.options || {};
  const functionName = packet.functionName;
  const kernelIp = socket.get('kernel_ip');
  const tenantId = socket.get('tenant');
  options.tenant_id = tenantId;
  abstractions.request(functionName, options, kernelIp)
  .then((result) => {
    socket.emit(`kernel:functions:${packet.id}`, result);
  })
  .catch((err) => {
    socket.emit(`error:${packet.id}`, err);
  });
};

/**
 * [kernelFunctionsHTTP: Get kernel functions by socket request]
 */
exports.kernelFunctionsHTTP = function kernelFunctions(req, res, next) {
  const options = req.body || {};
  const functionName = req.params.functionName;
  const kernelIp = req.user.tenant.kernel_ip;

  abstractions.request(functionName, options, kernelIp)
  .then((result) => {
    res.send(result);
  })
  .catch((err) => {
    next(err);
  });
};

exports.companyDashboard = function companyDashboard(req, res, next) {
  const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
  kernel.post('/kernelfunctions/defaultvalue', { tenant_id: req.user.tenant.database, property: 'EmpresaDef', valueOut: 'name' }, (err, response, body) => {
    if (err) return next(err);
    const entityName = body.response.data[0];
    const fechaFin = new Date().toISOString().substr(0, 10);
    var url = '';
    getDashboardURLHTTP(req, (error, response) => {
      for (var i=0; i<response.response.results.length; i++) {
        if (response.response.results[i].code === 'PanelEmpresa') {
          url = response.response.results[i].URL;
          break;
        }
      }
      url = url.replace(/&PARAMETERS/g, `&NombreEmpresa_1=${entityName}&Fecha_Fin_1=${fechaFin}`);
      var token = url.slice(url.indexOf('&pp='));
      url = url.slice(0,  url.indexOf('&pp='));
      url = url.slice(0, url.indexOf('=no')+3) + token + url.slice(url.indexOf('=no')+3);
      res.redirect(`${url}`);
    });
  });
};

exports.configurationButtonWS = function configurationButtonWS(socket, io, packet) {
  const options = {
    uri: `http://${socket.get('kernel_ip')}/configuration/configurationButton?tenant_id=${socket.get('tenant')}&username=${socket.store._id}`,
    method: 'GET',
    json: true,
  };
  request(options, (err, body, response) => {
    if (err) callback(socket.error(err));
    if (response.status !== 200) {
      const kernelError = { kernelError: response };
      callback(socket.error(kernelError));
    }
    socket.emit('configuration:button-receive', response)
  });
};