'use strict';

const debug = require('debug')('init');
const debugSck = require('debug')('socket');

const fsw = require('fs-walk');

const utils = require('../utils');
const protect = utils.protect;
const getDirectories = utils.getDirectories;

module.exports = (app, io, cb) => {
  // load modules
  const socketHandlers = [];
  let loaded = 0;
  const needed = getDirectories(__dirname);

  // walk through every folder inside this folder
  fsw.walk(__dirname, (root, dir, stat) => {
    if (stat.isDirectory()) {
      // this line loads the http endpoints for a specific module
      // and socketHandler now contains the returning function
      debug(`HTTP ${dir}`);

      let socketHandler = null;

      try {
        socketHandler = require(`${__dirname}/${dir}`)(app, protect);
      } catch (err) {
        debug('----------------------');
        debug('error loading', dir);
        debug(err);
        debug('----------------------');
      }

      if (typeof(socketHandler) === 'function') {
        // this line adds the http endpoints returning function
        // to the socketHandlers, because if there is a returning function
        // it will accept the socket and attach the socket handlers of that module
        socketHandlers.push({ dir, handler: socketHandler });
      }

      loaded++;
      if (loaded === needed) { cb(); }
    }
  });

  // on socket connections
  io.on('connection', (socket) => {
    socket.on('disconnect', () => {
      debugSck('disconnected', socket.get('email'));
    });
    for (const handle of socketHandlers) {
      debug(`WS ${handle.dir}`);
      try {
        handle.handler(socket, io);
      } catch (err) {
        debug('----------------------');
        debug('error loading', handle.dir);
        debug(err);
        debug('----------------------');
      }
    }
  });
};
