'use strict';

const db = require('../../database/mongo.js');
const BusinessApps = require('@intelisis/g4-runtime-api-business-apps');
const Catalogs = require('@intelisis/g4-runtime-api-catalog');
const Process = require('@intelisis/g4-process');
const JiraFacade = require('jira-facade');
const JIRA = new JiraFacade({
  protocol: 'https',
  host: 'intelisis.atlassian.net',
  port: 443,
  username: 'Support',
  password: 'support',
  apiVersion: 2,
});
// const _ = require('underscore');

const AWSSDK = require('aws-sdk');
const AWS = new AWSSDK.S3(__config.aws);

const exportFn = function exportFn(socket, io, pkg) {
  const { catalogName } = pkg;
  const tenantName = socket.get('tenant');
  const bapp = new BusinessApps({ db, tenantName });
  const proc = new Process({ }, { tenant: tenantName, db, socket, AWS, io });

  bapp.businessApps.findOne({ 'forms.name': catalogName }, { 'forms.$': 1 }).then((forms) => {
    if (!forms) {
      return socket.error(new Error(`Form ${catalogName} not found in any of the BusinessApp installed.`));
    }
    const form = forms.forms[0];
    const catalog = new Catalogs({ db, tenantName, json: { __catalog: catalogName } });
    return catalog.find().then((results) => {
      const newDatas = [];
      for (let result of results) {
        const newData = proc.kernelMapper(result.data, form);
        newData._id = String(result._id);
        newData.tenant_id = tenantName;
        newDatas.push(newData);
      }
      socket.emit('superSecretMaintenance:export', catalogName, newDatas);
    });
  }).catch((err) => {
    socket.error(err);
  });
};

const createTicket = function createTicket(req, res, next) {
  const { summary, description } = req.query;

  JIRA.createIssue({
    project: 'SIG',
    issueType: 'IT Help',
    summary: decodeURIComponent(summary).replace(/(?:\r\n|\r|\n)/g, '. '),
    description: decodeURIComponent(description),
    components: ['runtime'],
    assignee: 'azuniga',
    reporter: 'Support',
    priority: 'Medium',
    labels: ['runtime'],
  }, (err, issue) => {
    console.error(err);
    if (err) return next(err);
    res.end(`<script>document.write('<style type="text/css">* { font-family: Arial; }</style> La Incidencia <strong>${issue.key}</strong> fue reportada exitosamente.<br />Esta ventana se cierra automaticamente despues de 10 segundos.<br /><br /><span style="cursor: pointer; text-decoration: underline" onClick="window.close()">Cerrar ahora</span>'); setTimeout(window.close, 10000);</script>`);
  });
};

module.exports = (app) => {
  app.get('/maintenance/tickets/create', createTicket);

  return (socket, io) => {
    socket.on('superSecretMaintenance:export', exportFn.bind({}, socket, io));
  };
};
