'use strict';
const Movement = require('@intelisis/g4-runtime-api-catalog');

exports.getMovementsPerPage = (options) => {
  const ITEMS_PER_PAGE = options.ITEMS_PER_PAGE;
  const movementName = options.movementName;
  let page = options.page;
  page--;
  const skipValue = page * ITEMS_PER_PAGE;
  if (!movementName) {
    return new Promise((resolve, reject) => {
      reject(new Error('Movement name missing'));
    });
  }
  let select = options.fields || null;
  if (select) {
    select = select.split(',').join(' ');
  }
  const movement = new Movement({ __catalog: movementName, db: options.db, tenantName: options.tenantName, prefix: 'mov_' });
  const model = movement.catalog;
  return new Promise((resolve, reject) => {
    model.find().count().exec().then((count) => {
      model.find().select(select).skip(skipValue).limit(ITEMS_PER_PAGE).then((results) => {
        const mappedResults = results.map((item) => {
          const newItem = item;
          if (newItem.archived === false) {
            newItem.archived = '';
          }
          return newItem;
        });
        resolve({
          pagination: {
            items: count,
            perPage: ITEMS_PER_PAGE,
            page: page + 1,
            pages: Math.ceil(count / ITEMS_PER_PAGE),
          },
          results: mappedResults,
        });
      }).catch((err) => {
        reject(err);
      });
    });
  });
};

exports.getMovements = (options) => {
  const movement = new Movement({ __catalog: options.__catalog, db: options.db, tenantName: options.tenantName, prefix: 'mov_' });
  const model = movement.catalogs;
  return model.find().sort('name');
};

exports.customActions = {
  left: {
    cancelMovement: {
      left: [{
        status: 'on',
        icon: 'actionbar-cancel',
        callback: `(function() {
          const message = '¿Seguro que deseas cancelar este movimiento?'
          if (window.confirm(message)) {
            const movementChannel = Backbone.Radio.channel('movements');
            socket.emit('movements:cancel', { _id: "@documentId" });
            socket.on('movements:cancel', function(response) {
              if(response)
              {
                movementChannel.trigger('modal:close');
              }
            });
          }
        })()`,
      }],
    },
  },
};
