const db = require('../../database/mongo.js');
const mongoose = require('mongoose');
const BusinessApp = require('@intelisis/g4-runtime-api-business-apps');
const stepAccessSchema = require('../../database/schemas/step_access.js');
/**
 * find access
 *
 * @param      {<type>}  options  The options
 * @return     {<type>}  { description_of_the_return_value }
 */
const findUserAccess = options => {
  const { userId, tenantName } = options;
  return new Promise((resolve, reject) => {
    const tenantDb = db.useDb(tenantName);
    const Step = tenantDb.model('step_access', stepAccessSchema);
    return Step.find({
      $or: [{ 'users._id': userId }, { byOmission: userId }],
    })
    .lean()
    .select('-users')
    .exec((err, result) => {
      if (err) reject(err);
      resolve(result);
    });
  });
};
/**
 * Filter diagrams access
 * return only users that have acces on the init signal
 */
const filterAccess = (bapps, userId, tenantName) => {
  // First get userAccesses
  return findUserAccess({ userId, tenantName }).then(result => {
    const stepsAllowed = result;

    return bapps.map(bapp => {
      const bappId = bapp._id;
      // Check in each diagram init signal
      const diagramsAllowed = [];
      bapp.diagrams.map(diagram => {
        const diagramId = diagram._id;
        // Access to each diagram's step
        diagram.pools.map(pool => {
          pool.droppableZones.map(droppableZone => {
            droppableZone.elementsCollection.map(element => {
              const stepId = element._id;
              if (element.subtype === 'start') {
                // Check user accesses
                stepsAllowed.map(allow => {
                  if (String(allow.bappId) === String(bappId) && String(allow.diagramId) === String(diagramId) && String(allow.stepId) === String(stepId)) {
                    allow = false;
                    delete diagram.pools;
                    diagramsAllowed.push(diagram);
                  }
                  return allow;
                });
              }
              return element;
            });
            return droppableZone;
          });
          return pool;
        });
        return diagram;
      });
      bapp.diagrams = diagramsAllowed;
      return bapp;
    }).filter(bapp => {
      if (bapp.diagrams.length === 0) {
        return false;
      }
      return true;
    });
  });
};

exports.getBusinessApps = function getBusinessApp(options) {
  return new Promise((resolve, reject) => {
    const { tenantName, filter, userId } = options;
    const tenant = db.useDb(tenantName);
    const schema = tenant.model('business_apps', new mongoose.Schema({
      name: 'String',
      diagrams: [],
    }));

    const query = {
      name: 1,
      'diagrams.name': 1,
      'diagrams.marketplace.screenshots': 1,
      'diagrams._id': 1,
      name_lower: { $toLower: '$name' },
    };
    // Get diagram's pools info if filter
    if (filter) {
      query['diagrams.pools.droppableZones.elementsCollection._id'] = 1;
      query['diagrams.pools.droppableZones.elementsCollection.subtype'] = 1;
    }

    schema.aggregate([{
      $project: query,
    }, {
      $sort: {
        name_lower: 1,
      },
    }])
    .exec((err, results) => {
      if (err) return reject(err);
      let result = results;
      if (filter) {
        result = filterAccess(results, userId, tenantName);
      }
      return resolve(result);
    });
  });
};

exports.fetchDiagram = function fetchDiagram(options) {
  const { tenantName, bappId, diagramId } = options;
  return new Promise((resolve, reject) => {
    const bapp = new BusinessApp({ db, tenantName, _id: bappId });
    return bapp.fetch()
    .then(() => {
      const diagram = bapp.getDiagram(diagramId);
      return diagram.then(result => {
        resolve(result);
      })
      .catch(err => {
        reject(err);
      });
    })
    .catch(err => {
      reject(err);
    });
  });
};
