const debug = require('debug')('process');
// const Process = require('@intelisis/g4-runtime-api-process');
const db = require('../../database/mongo.js');
const BusinessApp = require('@intelisis/g4-runtime-api-business-apps');
// const mongoose = require('mongoose');
const abstractions = require('./abstractions');
const getBusinessApps = abstractions.getBusinessApps;
const fetchDiagram = abstractions.fetchDiagram;
const AWSSDK = require('aws-sdk');
const AWS = new AWSSDK.S3(__config.aws);
const Process = require('@intelisis/g4-process');

exports.getMetadataList = function getMetadataListV2(req, res, next) {
  // const tenant = db.useDb(req.tenant);
  // const schema = tenant.model('business_apps', new mongoose.Schema({
  //   name: 'String',
  // }));
  // schema.find({}).select('name').exec((err, results) => {
  //   if (err) return next(err);
  //   return res.json(results);
  // });
};

exports.startProcess = function startProcess(socket, io, packet) {
  // the packet contains BApp Id and diagram id
  // so parse them
  const bappId = packet.bappId;
  const diagramId = packet.diagramId;

  // TODO: check if user has permission to launch process
  debug('start:request', bappId, diagramId);
  const bapp = new BusinessApp({ db, tenantName: socket.get('tenant'), _id: bappId });
  bapp.fetch().then(() => {
    debug('BusinessApp', bapp.get('name'));
    return bapp.getDiagram(diagramId).then((diagram) => {
      debug('Diagram', diagram.name);
      const tenant = socket.get('tenant');

      const proc = new Process(null, { tenant, db, AWS, socket, io });
      proc.set('diagram', diagram);
      proc.set('bappId', bappId);
      return proc.next().then(() => {
        debug('Process started');
      }).catch((err) => {
        socket.error(err);
      });
    });
  }).catch((err) => {
    socket.error(err);
  });
};
/**
 * Gets the business apps list ws.
 *
 * @param      {<type>}  socket  The socket
 */
exports.getBusinessAppsListWS = function fetchBusinessAppsList(socket) {
  const tenantName = socket.get('tenant');
  getBusinessApps({ tenantName, filter: false })
  .then((result) => {
    socket.emit('business_apps:list', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};
/**
 * Gets the business apps list http.
 *
 * @param      {<type>}    req     The request
 * @param      {<type>}    res     The resource
 * @param      {Function}  next    The next
 */
exports.getBusinessAppsListHTTP = function getBusinessAppsList(req, res, next) {
  const tenantName = req.tenant;
  getBusinessApps({ tenantName, filter: false })
  .then((result) => {
    res.send(result);
  })
  .catch((err) => {
    next(err);
  });
};
/**
 * Gets the business apps list ws.
 *
 * @param      {<type>}  socket  The socket
 */
exports.getBusinessAppsListWS_access = function fetchBusinessAppsList(socket) {
  const tenantName = socket.get('tenant');
  const userId = socket.get('user')._id;
  getBusinessApps({ tenantName, userId, filter: true })
  .then((result) => {
    socket.emit('business_apps:list', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};
/**
 * Gets the business apps accessible for session
 *
 * @param      {<type>}    req     The request
 * @param      {<type>}    res     The resource
 * @param      {Function}  next    The next
 */
exports.getBusinessAppsListHTTP_access = function getBusinessAppsList(req, res, next) {
  const tenantName = req.tenant;
  const userId = req.user._id;
  getBusinessApps({ tenantName, userId, filter: true })
  .then((result) => {
    res.send(result);
  })
  .catch((err) => {
    next(err);
  });
};
/**
 * Gets the diagram.
 *
 * @param      request    req     The request
 * @param      response    res     The resource
 * @param      set errors  next    The next
 */
exports.getDiagram = function getDiagram(req, res, next) {
  const tenantName = req.tenant;
  const bappId = req.params._id;
  const diagramId = req.params.diagram_id;
  fetchDiagram({ tenantName, bappId, diagramId })
  .then(result => {
    res.send(result);
  })
  .catch(err => {
    next(err);
  });
};
