'use strict';
const ITEMS_PER_PAGE = 25;
/**
 * Get all reports from kernel
 * @param  {String} kernelIp
 * @param  {String} tenantName
 * @return {[Object]}
 */
exports.getReports = (options) => {
  const { kernelIp, tenantName } = options;
  return new Promise((resolve, reject) => {
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.get(`/subtypes?tenant_id=${tenantName}&type=report`, (err, response, body) => {
      if (err) return reject(err);
      if (body.status !== 200) return reject(new Error('Kernel Error'));
      resolve(body.response);
    });
  });
};

/**
 * Get paginates reports by name from kernel
 * @param  {String} kernelIp
 * @param  {String} tenantName
 * @param  {String} fields
 * @param  {String} report
 * @param  {Number} page
 * @return {[Object]}
 */
exports.getReportsItemsPerPage = (options) => {
  const { kernelIp, tenantName, fields, report, page, user, roles } = options;
  return new Promise((resolve, reject) => {
    if (!fields || !report || !page) {
      const error = new Error('page or catalog missing');
      return reject(error);
    }
    const url = `/subtypes/records?tenant_id=${tenantName}&kind=Normal&fields=${fields},iv,salt,passphrase,key_size,iteration&type=report&subType=${report}&user=${user}&roles=${roles}&page=${page}&limit=${ITEMS_PER_PAGE}`;
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.get(url, (err, response, body) => {
      if (err) return reject(err);
      if (body.status !== 200) return reject(new Error('Kernel Error'));
      resolve(body.response);
    });
  });
};

/**
 * Get specific reports by _id from kernel
 * @param  {String} kernelIp
 * @param  {String} reportId
 * @param  {String} tenantName
 * @return {[Object]}
 */
exports.getReport = (options) => {
  const { kernelIp, reportId, tenantName } = options;

  return new Promise((resolve, reject) => {
    const kernel = require('request-json').createClient(`http://${kernelIp}/`);
    return kernel.get(`/catalogs/${tenantName}/${reportId}`, (err, response, body) => {
      if (err) return reject(err);
      if (body.status !== 200) return reject(new Error('Kernel Error'));
      const name = body.response.name;
      const url = body.response.properties.URL;
      resolve({ name, url });
    });
  });
};
