'use strict';
/**
 * [abstractions: Useful functions]
 * @type {Object}
 */
const abstractions = require('./abstractions');
const getReportsItemsPerPage = abstractions.getReportsItemsPerPage;
const getReports = abstractions.getReports;
const getReport = abstractions.getReport;
const request = require('request');
/**
 * Get list of reports by http request
 */
exports.getReportListHTTP = (req, res, next) => {
  const options = {
    tenantName: req.tenant,
    kernelIp: req.user.tenant.kernel_ip,
  };
  getReports(options)
  .then((result) => {
    res.send(result);
  }).catch((err) => {
    next(err);
  });
};

/**
 * [getReportListWS: Get list of reports by socket request]
 */
exports.getReportListWS = (socket) => {
  const options = {
    tenantName: socket.get('tenant'),
    kernelIp: socket.get('kernel_ip'),
  };
  getReports(options)
  .then((result) => {
    socket.emit('reports:list', result);
  }).catch((err) => {
    socket.error(err);
  });
};

/**
 * [getReportsItemsPerPageHTTP: Get report's items by (http request)]
 */
exports.getReportsItemsPerPageHTTP = (req, res, next) => {
  const options = {
    tenantName: req.tenant,
    kernelIp: req.user.tenant.kernel_ip,
    fields: req.query.fields,
    report: req.query.catalog,
    page: req.query.page,
    user: req.user.accounts[0].email,
    roles: req.user.roles,
  };
  getReportsItemsPerPage(options)
  .then((result) => {
    res.send(result);
  })
  .catch((err) => {
    next(err);
  });
};

/**
 * [getReportsItemsPerPageWS: Get report's items by (socket request)]
 */
exports.getReportsItemsPerPageWS = (socket, io, packet) => {
  const options = {
    tenantName: socket.get('tenant'),
    kernelIp: socket.get('kernel_ip'),
    fields: '_id,code%20as%20Clave,name%20as%20Nombre,description%20as%20Descripcion,kind%20as%20Tipo,status%20as%20Estatus,class%20as%20Clase,fnCatProperty(cat._id,%20%22URL%22)%20AS%20URL',
    // fields: packet.fields,
    report: packet.report,
    page: packet.page,
  };
  getReportsItemsPerPage(options)
  .then((result) => {
    socket.emit('reports:items', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};

/**
 * [getReport: Get scpecific report by socket request]
 */
exports.getReport = (socket, io, packet) => {
  const options = {
    kernelIp: socket.get('kernel_ip'),
    reportId: packet._id,
    tenantName: socket.get('tenant')
  };
  getReport(options)
  .then((result) => {
    socket.emit('reports:show', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};

exports.getReportURL = function getReportURL(socket, packet, callback) {
  var docId = '';
  if (packet._documentId !== undefined) { docId = `&document_id=${packet._documentId}`;}
  const options = {
    uri: `http://${socket.get('kernel_ip')}/reports/records?tenant_id=${socket.get('tenant')}&type=${packet._type}&process=${packet._process}&step=${packet._step}${docId}&user=${socket.store.email}&roles=${socket.store.user.roles}&fields=fnCatProperty(cat._id,%22URL%22)%20as%20URL,label,iv,salt,passphrase,key_size,iteration`,
    method: 'GET',
    json: true,
  };
  request(options, (err, body, response) => {
    if (err) callback(socket.error(err));
    if (response.status !== 200) {
      const kernelError = { kernelError: response };
      callback(socket.error(kernelError));
    }
    var string = '';
    for (var i=0; i<response.response.results.length; i++) {
      string += JSON.stringify(response.response.results[i]);
      if (i+1 < response.response.results.length) {
        string += ',';
      }
    }
    var action = '[' + string.replace(/"`/g, "`").replace(/`"/g, "`") + ', {},]';
    callback(null,action);
  });
};

exports.getDashboardURLWS = function getDashboardURL(socket, callback) {
  const options = {
    uri: `http://${socket.get('kernel_ip')}/subtypes/records?tenant_id=${socket.get('tenant')}&type=report&kind=Dashboard&user=${socket.store.email}&roles=${socket.store.user.roles}&fields=code,fnCatProperty(cat._id,%22URL%22)%20as%20URL,iv,salt,passphrase,key_size,iteration`,
    method: 'GET',
    json: true,
  };
  request(options, (err, body, response) => {
    if (err) callback(socket.error(err));
    if (response.status !== 200) {
      const kernelError = { kernelError: response };
      callback(socket.error(kernelError));
    }
    callback(null,response);
  });
};

exports.getDashboardURLHTTP = function getDashboardURL(req, callback) {
  const options = {
    uri: `http://${req.user.tenant.kernel_ip}/subtypes/records?tenant_id=${req.user.tenant.database}&type=report&kind=Dashboard&user=${req.user.accounts[0].email}&roles=${req.user.roles}&fields=code,fnCatProperty(cat._id,%22URL%22)%20as%20URL,iv,salt,passphrase,key_size,iteration`,
    method: 'GET',
    json: true,
  };
  request(options, (err, body, response) => {
    if (err) callback(reject(err));
    if (response.status !== 200) {
      const kernelError = { kernelError: response };
      callback(socket.error(kernelError));
    }
    callback(null,response);
  });
};