'use strict';
const mongoose = require('mongoose');
const API_AUTH = 'http://auth.g4.intelisis.co/v1';
const auth = require('request-json').createClient(API_AUTH);
mongoose.Promise = require('bluebird');
const stepAccessSchema = require('../../database/schemas/step_access.js');
const db = require('../../database/mongo.js');
/**
 * Sorter by object params eg 'key.key2'
 * @param  Array arr   Array objects to sort
 * @param  String order
 * @return Array
 */
const sorter = (arr, order) => {
  const keys = order.split('.');
  let ordering;
  keys.map(key => {
    ordering = [key];
  });
  arr.sort((a, b) => {
    const keyA = a.ordering;
    const keyB = b.ordering;
    // Compare the 2 keys
    if (keyA < keyB) return -1;
    if (keyA > keyB) return 1;
    return 0;
  });
  return arr;
};
/**
 * Saves an access.
 *
 * @param      Object   options  The options
 * @return     {Promise}
 */
exports.saveAccess = options => {
  const { bappId, diagramId, stepId, tenant, users, byOmission } = options;
  return new Promise((resolve, reject) => {
    const tenantDb = db.useDb(tenant.database);
    const Step = tenantDb.model('step_access', stepAccessSchema);
    const removeAccess = [];
    /**
     * Filter only user with access to then upsert,
     * and collect user to remove access
     */
    const access = users.filter(user => {
      if (!user.access) {
        removeAccess.push(user._id);
        return false;
      }
      return true;
    });
    const query = {
      stepId,
      bappId,
      diagramId,
    };
    const upsertAccess = {
      $addToSet: {
        users: { $each: access },
      },
      byOmission,
    };
    const revokeAccess = {
      $pull: {
        users: { $in: removeAccess },
      },
    };
    return Step.findOneAndUpdate(query, revokeAccess)
    // First upsert user's access
    .exec((err) => {
      if (err) return reject(err);
      return Step.findOneAndUpdate(query, upsertAccess, { upsert: true, new: true })
      .exec((errRevoke, result) => {
        if (errRevoke) return reject(errRevoke);
        resolve(result);
      });
    });
  });
};
/**
 * Gets the access.
 *
 * @param      {<type>}   options  The options
 * @return     {Promise}
 */
exports.getAccess = options => {
  const { bappId, diagramId, stepId, tenant, userId } = options;
  return new Promise((resolve, reject) => {
    // Get tenant's users
    auth.get(`/v1/tenants/${tenant._id}`, (err, response, body) => {
      let tenantUsers = body.users;
      tenantUsers = tenantUsers.map(user => {
        return {
          _id: user._id,
          details: user.details,
          access: false,
        };
      });
      // Get steps's users
      const tenantDb = db.useDb(tenant.database);
      const Step = tenantDb.model('step_access', stepAccessSchema);
      Step.findOne({ bappId, diagramId, stepId })
      .lean()
      .exec((errQuery, result) => {
        if (errQuery) return reject(errQuery);
        if (!result || !result.users) return resolve({ users: tenantUsers, byOmission: userId });

        const usersAcess = result.users;
        let users = tenantUsers.map(tenantUser => {
          usersAcess.filter(userAccess => {
            if (String(tenantUser._id) === String(userAccess._id)) {
              tenantUser.access = userAccess.access;
            }
            return false;
          });
          return tenantUser;
        });
        users = sorter(users, 'details.firstName');
        resolve({ users, byOmission: result.byOmission });
      });
    });
  });
};
