'use strict';
/**
 * [Dependencies]
 */
const request = require('request');
const API_AUTH = 'http://auth.g4.intelisis.co/v1';
const session = require('express-session');
const connectRedis = require('connect-redis');
const RedisStore = connectRedis(session);
const store = new RedisStore(__config.session.redis);

/**
 * [showTenants: get all available tenants for request session]
 */
exports.showTenants = function showTenants(socket) {
  const cookie = socket.handshake.headers.cookie;
  const requestOptions = {
    url: `${API_AUTH}/user/tenants?_id=${socket.get('_id')}`,
    json: true,
    headers: {
      cookie,
    },
  };
  request(requestOptions, (err, response, body) => {
    if (err) return socket.error(err);
    if (!body.tenants) {
      const error = new Error('No tenants found');
      return socket.error(error);
    }
    const tenants = body.tenants.map((tenant) => {
      const tenantParsed = {
        _id: tenant._id,
        name: tenant.name,
        image: tenant.image,
      };
      return tenantParsed;
    });
    return socket.emit('tenants:show', tenants);
  });
};

/**
 * [selectTenant: change tenant for request session]
 */
exports.selectTenant = function selectTenant(socket, io, packet) {
  const cookie = socket.handshake.headers.cookie;
  const tenantId = packet.tenantId;
  const sessionId = socket.handshake.query.sessionId;
  const requestOptions = {
    url: `${API_AUTH}/user/tenants?_id=${socket.get('_id')}`,
    json: true,
    headers: {
      cookie,
    },
  };
  request(requestOptions, (err, response, body) => {
    if (err) return socket.error(err);
    if (!body.tenants) {
      const error = new Error('No tenants found');
      return socket.error(error);
    }
    let targetTenant = false;
    body.tenants.map((tenant) => {
      if (String(tenant._id) === String(tenantId)) {
        targetTenant = tenant;
        delete targetTenant.image;
      }
      return tenant;
    });
    if (!targetTenant) {
      const error = new Error('Tenant not found');
      return socket.error(error);
    }
    return store.get(sessionId, (error, userSession) => {
      if (error) {
        return socket.error(new Error('InternalServerError'));
      }
      userSession.passport.user.tenant = targetTenant;
      return store.set(sessionId, userSession, (errSet) => {
        if (err) return socket.error(errSet);
        socket.set('tenant', userSession.passport.user.tenant.database);
        socket.set('kernel_ip', userSession.passport.user.tenant.kernel_ip || '');
        return socket.emit('tenants:select', { success: true });
      });
    });
  });
};

/**
 * [isTenantSelected: show by boolean field if the request session have a tenent selected]
 */
exports.isTenantSelected = function isTenantSelected(socket) {
  let response = false;
  const tenant = socket.get('tenant');
  if (tenant) response = true;
  return socket.emit('tenants:selected', response);
};
