'use strict';

const responseTime = require('response-time');
const corser = require('corser');
const bodyParser = require('body-parser');
const compression = require('compression');
const express = require('express');
const session = require('express-session');
const connectRedis = require('connect-redis');
const debug = require('debug')('http');
const onHeaders = require('on-headers');
const shortid = require('shortid');

const RedisStore = connectRedis(session);
const app = express();

// show response time in every response
app.use(responseTime());

// set cors policies
app.use(corser.create(__config.express.policies.cors));

// start session middleware
const options = __config.session.options;
options.store = new RedisStore(__config.session.redis);
app.use(session(options));

// add sessions user to req.user
app.use((req, res, next) => {
  if (req.session.passport) {
    req.user = req.session.passport.user;
  }
  if (req.user && req.user.tenant) {
    req.tenant = req.user.tenant.database;
  }
  next();
});

app.use((req, res, next) => {
  const tenant = req.tenant;
  req.random = shortid.generate();
  let email;
  if (req.user && req.user.accounts.length > 0) {
    email = req.user.accounts[0].email;
  }

  onHeaders(res, () => {
    debug(req.method + ' ✓ ' + req.originalUrl, email, tenant, req.random);
  });
  debug(req.method + ' … ' + req.originalUrl, email, tenant, req.random);

  next();
});

app.disable('x-powered-by');

// show api version and dependency status in every response
app.use((req, res, next) => {
  res.header('X-API-Version', __config.version);
  res.header('X-API-Dependencies', JSON.stringify(dependencies));
  next();
});

// use compression
app.use(compression());

// set bodyParser to limit 100mb requests
app.use(bodyParser.json({ limit: '100mb' }));

module.exports = app;
