const debug = require('debug')('init');

// 3rd party modules
const sio = require('@intelisis/socket.io');
const adapter = require('socket.io-redis');
const redis = require('redis').createClient;

const socketIOPassport = require('./security/');

module.exports = (httpListenCallback) => {
  // express app & middleware
  const app = require('./express/');
  debug('Express Server');

  // listen to http
  const server = app.listen(__config.express.port, httpListenCallback);

  // start listening to websocket
  // use socket.io-redis for distribution of sockets
  // non-local
  const io = sio(server);

  const pub = redis(__config.socket.redis.port, __config.socket.redis.host, { auth_pass: __config.socket.redis.pass });
  const sub = redis(__config.socket.redis.port, __config.socket.redis.host, { return_buffers: true, auth_pass: __config.socket.redis.pass });

  io.set('origins', '*:*');
  io.adapter(adapter({ pubClient: pub, subClient: sub }));

  // use passport on socket.io
  io.use(socketIOPassport());
  debug('socket.io Server');

  // load database connection
  require('./database/mongo.js');

  // load api endpoints
  require('./endpoints/loader.js')(app, io, () => {
    // when all other endpoints got loaded, load error fallback
    require('./express/errors.js')(app);
  });
};

process.on('uncaughtException', (err) => {
  const message = err.message;
  debug(err);
  if (message.indexOf('Redis') > -1) {
    debug(err);
  } else {
    process.exit(0);
  }
});
