'use strict';

const ambMap = {
  'g4-devel': 'Intelisis_Dev',
  'g4-qa': 'Intelisis_QA',
  'g4-prod': 'Intelisis_REL',
};

const ambName = ambMap[process.env.NODE_ENV] || 'Intelisis_Dev';

const getReportURL = require('./endpoints/reports/functions').getReportURL;
const getDashboardURLWS = require('./endpoints/reports/functions').getDashboardURLWS;
const fs = require('fs');
const path = require('path');
const _ = require('underscore');
const request = require('request');

exports.getDirectories = (srcpath) => {
  const dirs = fs.readdirSync(srcpath).filter((file) => fs.statSync(path.join(srcpath, file)).isDirectory());
  return dirs;
};

exports.protect = (req, res, next) => {
  if (req.tenant) {
    next();
  } else {
    next(new Error('Unauthorized'));
  }
};

/*
  Actions, ActionBar
*/
const actionsLeft = [];
const actionsRight = [{
  status: 'on',
  icon: 'actionbar-abort',
  callback: `(function() {
    const catalogChannel = Backbone.Radio.channel('catalogs');
    catalogChannel.trigger('modal:close');
  })()`,
}];

const actionsMovRight = [{
  status: 'on',
  icon: 'actionbar-abort',
  callback: `(function() {
    const movementChannel = Backbone.Radio.channel('movements');
    movementChannel.trigger('modal:close');
  })()`,
}];


const actionIconMap = (label) => {
  let iconClass = '';
  if (label === 'Modificar') {
    iconClass = 'pencil';
  }
  if (label === 'Eliminar') {
    iconClass = 'trash';
  }
  if (label.indexOf('Revision') > -1) {
    iconClass = 'eye';
  }
  if (label.indexOf('Archivar') > -1) {
    iconClass = 'clock-o';
  }
  if (iconClass) {
    return `fa fa-${iconClass}`;
  }
  return '';
};

exports.createMovActions = (status, documentName, documentId, signals, singleSignal, presetActions) => {
  let actionsCenter = [];

  let affectStatus;

  if(status === 'CANCELADO')
    affectStatus = 'off';
  else
    affectStatus = 'on';

  if(signals.length > 0){
    _.each(signals, (signal) => {
      const fnStr = `(function () {
        const movementChannel = Backbone.Radio.channel('movements');
        const signalId = '${signal._id}';

        socket.emit('affectMov:signal', {
          signalId:  signalId,
          documentName: '${documentName}',
          documentId: '${documentId}',
        });

        movementChannel.trigger('modal:close');
      })();`;

      actionsCenter.push({
        status: affectStatus,
        icon: actionIconMap(signal.extra.label),
        label: `&nbsp; ${signal.extra.label}`,
        callback: fnStr,
      });
    });
  }
  else{
    // const fnStr = `(function () {
    //   const movementChannel = Backbone.Radio.channel('movements');
    //   const processId = '${singleSignal.processId}';
    //   socket.emit('affectProcess:signal', {
    //     processId:  processId,
    //     documentName: '${documentName}',
    //     documentId: '${documentId}',
    //   });
    //   movementChannel.trigger('modal:close');
    // })();`;
    const fnStr = ``;

    actionsCenter.push({
      status: 'off',
      icon: 'actionbar-apply',
      className: 'apply',
      callback: fnStr,
    });
  }


  actionsCenter = _(actionsCenter).sortBy('label').reverse();

  const actions = {};
  actions.left = actionsLeft;
  actions.center = [{
    status: affectStatus,
    icon: 'actionbar-apply',
    className: 'apply',
    actions: actionsCenter,
  }];
  actions.right = actionsMovRight;
  if (presetActions && Array.isArray(presetActions)) {
    // Over write preset actions
    presetActions.map((action) => {
      _.extend(actions, action);
    });
  }
  return actions;
};

exports.createActions = (signals, presetActions) => {
  let actionsCenter = [];
  _.each(signals, (signal) => {
    const fnStr = `(function () {
      const catalogChannel = Backbone.Radio.channel('catalogs');
      const catalogName = catalogChannel.request('catalogName');
      const catalogItemId = catalogChannel.request('catalogItemId');
      const signalId = '${signal._id}';

      socket.emit('affect:signal', {
        signalId:  signalId,
        catalogName: catalogName,
        catalogItemId: catalogItemId,
      });

      catalogChannel.trigger('modal:close');
    })();`;
    actionsCenter.push({
      status: 'on',
      icon: actionIconMap(signal.extra.label),
      label: `&nbsp; ${signal.extra.label}`,
      callback: fnStr,
    });
  });

  actionsCenter = _(actionsCenter).sortBy('label').reverse();

  const actions = {};
  actions.left = actionsLeft;
  actions.center = [{
    status: 'on',
    icon: 'actionbar-apply',
    className: 'apply',
    actions: actionsCenter,
  }];

  actions.right = actionsRight;
  if (presetActions && Array.isArray(presetActions)) {
    // Over write preset actions
    presetActions.map((action) => {
      _.extend(actions, action);
    });
  }
  return actions;
};

// Se usa en los documentos cuando se abren desde el inbox
exports.customActions = (socket, payload, options, callback) => {
  const { entityName, formName, tenantName, entityId, processId, stepId, stepLabel } = options;
  // Define origin
  const packetStep = stepId ? JSON.stringify({ stepId }) : JSON.stringify({ processId });
  const tenant = socket.get('tenant');
  const subType = 'Nomina';
  const type = 'report';
  getReportURL(socket, { _type: type, _process: formName, _step: stepLabel, _documentId: payload.State.document_id }, (error, response) => {
    if (error) { callback(error); }
    const returnValue = {
      left: [{
        icon: 'actionbar-information',
        className: 'information',
        status: 'on',
        popover: true,
        actions: eval(response),
      }, {
        icon: 'actionbar-print',
        className: 'print',
        status: 'off',
      }],
      right: [{
        icon: 'actionbar-attachments',
        className: 'attachments',
        status: 'off',
      }, {
        icon: 'actionbar-copy',
        className: 'copy',
        status: 'off',
      }, {
        icon: 'actionbar-share',
        className: 'share',
        status: 'off',
      }, {
        status: 'on',
        icon: 'actionbar-more',
        className: 'apply',
        popover: true,
        actions: [
          {
            popover: true,
            status: 'on',
            icon: '',
            label: '&nbsp; Asignar Accesos',
            callback: `(function(){
              socket.emit('inbox:access', ${packetStep});
            })()`,
          }, {},
        ],
      }, 

      // Se comenta el boton de Abort ya que no se requiere para el "Inbox".
      /*{
        status: 'on',
        icon: 'actionbar-abort',
        callback: `(function() {
          const catalogChannel = Backbone.Radio.channel('catalogs');
          catalogChannel.trigger('modal:close');
        })()`,
      }*/],

    };

    callback(null, returnValue);
  });
};

// Se usa en los catálogos cuando se abren desde el modal
exports.customCatActions = (socket, options, callback) => {
  const { entityName, formName, tenantName, entityId, processId, stepId } = options;
  // Define origin
  const packetStep = stepId ? JSON.stringify({ stepId }) : JSON.stringify({ processId });
  const returnValue = {
    left: [{
      icon: 'actionbar-information',
      className: 'information',
      status: 'off',
    }, {
      icon: 'actionbar-print',
      className: 'print',
      status: 'off',
    }],
    right: [{
      icon: 'actionbar-attachments',
      className: 'attachments',
      status: 'off',
    }, {
      icon: 'actionbar-copy',
      className: 'copy',
      status: 'off',
    }, {
      icon: 'actionbar-share',
      className: 'share',
      status: 'off',
    }, {
      status: 'on',
      icon: 'actionbar-more',
      className: 'apply',
      popover: true,
      actions: [
        {
          popover: true,
          status: 'on',
          icon: '',
          label: '&nbsp; Asignar Accesos',
          callback: `(function(){
            socket.emit('inbox:access', ${packetStep});
          })()`,
        }, {},
      ],
    }, {
      status: 'on',
      icon: 'actionbar-abort',
      callback: `(function() {
        const catalogChannel = Backbone.Radio.channel('catalogs');
        catalogChannel.trigger('modal:close');
      })()`,
    }],
  };

  if (entityId) {
    returnValue.left.push({
      status: 'on',
      icon: 'actionbar-cancel',
      callback: `(function() {
        const message = '¿Seguro que deseas eliminar este catalogo?'
        const catalogsChannel = Backbone.Radio.channel('catalogs');
        const catalogGridChannel = Backbone.Radio.channel('catalog-grid');
        if (window.confirm(message)) {
          socket.emit('catalogs:delete', { _id: '${entityId}' });
          socket.on('catalogs:delete', function() {
            catalogGridChannel.request('fetch');
            catalogGridChannel.trigger('fetch');
            catalogsChannel.trigger('modal:close');
          });
        }
      })()`,
    });
  }

  getDashboardURLWS(socket, (error, response) => {
    if (error) { callback(error); }
    var url = '';
    var show = 0;
    if (formName === 'Personal') {
      for (var i=0; i<response.response.results.length; i++) {
        if (response.response.results[i].code === 'PanelPersonal') {
          url = response.response.results[i].URL;
          break;
        }
      }
      url = url.replace(/&PARAMETERS/g, `&tenant=${tenantName}&entity=${entityId}`);
      show = 1;
    }
    else if (formName === 'Empresa') {
      for (var i=0; i<response.response.results.length; i++) {
        if (response.response.results[i].code === 'PanelEmpresa') {
          url = response.response.results[i].URL;
          break;
        }
      }
      const fechaFin = new Date().toISOString().substr(0, 10);
      url = url.replace(/&PARAMETERS/g, `&NombreEmpresa_1=${entityName}&Fecha_Fin_1=${fechaFin}`);
      show = 1;
    }

    var token = url.slice(url.indexOf('&pp='));
    url = url.slice(0,  url.indexOf('&pp='));
    url = url.slice(0, url.indexOf('=no')+3) + token + url.slice(url.indexOf('=no')+3);
    if (show === 1) {
      returnValue.left.push({
        status: 'on',
        icon: 'actionbar-information',
        className: 'information',
        callback: `(function () {
          window.open('${url}', '_blank');
        })();`,
      });
    }

    callback(null, returnValue);
  });
};

const movCancel = (socket, packet, callback) => {
  const options = {
    uri: `http://${socket.get('kernel_ip')}/configuration/cancelled?tenant_id=${socket.get('tenant')}&documentId=${packet._documentId}&status=${packet._status}&cancelStatus=${packet._cancelStatus}`,
    method: 'GET',
    json: true,
  };
  request(options, (err, body, response) => {
    if (err) return callback(socket.error(err));
    if (response.status !== 200) {
      const kernelError = { kernelError: response };
      return callback(socket.error(kernelError));
    }
    var string = '';
    for (var i=0; i<response.response.results.length; i++) {
      string += JSON.stringify(response.response.results[i]);
      if (i+1 < response.response.results.length) {
        string += ',';
      }
    }
    var action = '[' + string.replace(/"`/g, "`").replace(/`"/g, "`") + ']';
    return callback(null,action);
  });
};

// Se usa en los documentos cuando se abren desde el modal
exports.customMovActions = (socket, options, callback) => {
  const { documentId, formName, tenantName, entityName, processId, stepId, status, stepLabel } = options;
  const type = 'report';

  let cancelStatus;

  if(status === 'CANCELADO')
    cancelStatus = 'off';
  else
    cancelStatus = 'on';

  // Define origin
  const packetStep = stepId ? JSON.stringify({ stepId }) : JSON.stringify({ processId });
  const returnValue = {
    left: [{
      icon: 'actionbar-information',
      className: 'information',
      status: 'off',
    }, {
      icon: 'actionbar-print',
      className: 'print',
      status: 'off',
    }, {
      icon: 'actionbar-undo',
      className: 'undo',
      status: 'off',
    }],
    right: [{
      icon: 'actionbar-attachments',
      className: 'attachments',
      status: 'off',
    }, {
      icon: 'actionbar-copy',
      className: 'copy',
      status: 'off',
    }, {
      icon: 'actionbar-share',
      className: 'share',
      status: 'off',
    }, {
      status: 'on',
      icon: 'actionbar-more',
      className: 'apply',
      popover: true,
      actions: [
        {
          popover: true,
          status: 'on',
          icon: '',
          label: '&nbsp; Asignar Accesos',
          callback: `(function(){
            socket.emit('inbox:access', ${packetStep});
          })()`,
        }, {},
      ],
    }, {
      status: 'on',
      icon: 'actionbar-abort',
      callback: `(function() {
        const catalogChannel = Backbone.Radio.channel('movements');
        catalogChannel.trigger('modal:close');
      })()`,
    }],
  };

  getReportURL(socket, { _type: type, _process: formName, _step: stepLabel, _documentId: documentId }, (error, response) => {
    if (error) { callback(error); }
    returnValue.left[0] = 
    {
      icon: 'actionbar-information',
      className: 'information',
      status: 'on',
      popover: true,
      actions: eval(response),
    };
    if (documentId) {
      movCancel(socket, { _documentId: documentId, _status: status, _cancelStatus: cancelStatus }, (error, response) => {
        if (error) { callback(error); }
        returnValue.left.push({
          status: cancelStatus,
          icon: 'actionbar-cancel',
          popover: true,
          actions: eval(response),
        });
        callback(null, returnValue);
      });
    }
  });

};
